@@ -41,7 +41,6 @@ import java.awt.image.RenderedImage;
 import java.awt.image.WritableRaster;
 import java.io.IOException;
 import java.util.ArrayList;
-import java.util.Iterator;
 import java.util.List;
 
 import javax.imageio.IIOException;
@@ -132,6 +131,7 @@ public class JPEGImageWriter extends ImageWriter {
     private int newAdobeTransform = JPEG.ADOBE_IMPOSSIBLE;  // Change if needed
     private boolean writeDefaultJFIF = false;
     private boolean writeAdobe = false;
+    private boolean invertCMYK = false;
     private JPEGMetadata metadata = null;
 
     private boolean sequencePrepared = false;
@@ -655,6 +655,7 @@ public class JPEGImageWriter extends ImageWriter {
         newAdobeTransform = JPEG.ADOBE_IMPOSSIBLE;  // Change if needed
         writeDefaultJFIF = false;
         writeAdobe = false;
+        invertCMYK = false;
 
         // By default we'll do no conversion:
         int inCsType = JPEG.JCS_UNKNOWN;
@@ -808,6 +809,14 @@ public class JPEGImageWriter extends ImageWriter {
                                 }
                             }
                             break;
+                         case ColorSpace.TYPE_CMYK:
+                             outCsType = JPEG.JCS_CMYK;
+                             if (jfif != null) {
+                                 ignoreJFIF = true;
+                                 warningOccurred
+                                 (WARNING_IMAGE_METADATA_JFIF_MISMATCH);
+                             }
+                             break;
                         }
                     }
                 } // else no dest, metadata, not an image.  Defaults ok
@@ -1014,6 +1023,11 @@ public class JPEGImageWriter extends ImageWriter {
             System.out.println("outCsType: " + outCsType);
         }
 
+        invertCMYK =
+            (!rasterOnly &&
+             ((outCsType == JPEG.JCS_YCCK) ||
+              (outCsType == JPEG.JCS_CMYK)));
+
         // Note that getData disables acceleration on buffer, but it is
         // just a 1-line intermediate data transfer buffer that does not
         // affect the acceleration of the source image.
@@ -1374,7 +1388,7 @@ public class JPEGImageWriter extends ImageWriter {
     /**
      * Collect all the scan info from the given metadata, and
      * organize it into the scan info array required by the
-     * IJG libray.  It is much simpler to parse out this
+     * IJG library.  It is much simpler to parse out this
      * data in Java and then just copy the data in C.
      */
     private int [] collectScans(JPEGMetadata metadata,
@@ -1382,9 +1396,7 @@ public class JPEGImageWriter extends ImageWriter {
         List<SOSMarkerSegment> segments = new ArrayList<>();
         int SCAN_SIZE = 9;
         int MAX_COMPS_PER_SCAN = 4;
-        for (Iterator<MarkerSegment> iter = metadata.markerSequence.iterator();
-             iter.hasNext();) {
-            MarkerSegment seg = iter.next();
+        for (MarkerSegment seg : metadata.markerSequence) {
             if (seg instanceof SOSMarkerSegment) {
                 segments.add((SOSMarkerSegment) seg);
             }
@@ -1724,6 +1736,12 @@ public class JPEGImageWriter extends ImageWriter {
                                         srcBands);
         }
         raster.setRect(sourceLine);
+        if (invertCMYK) {
+            byte[] data = ((DataBufferByte)raster.getDataBuffer()).getData();
+            for (int i = 0, len = data.length; i < len; i++) {
+                data[i] = (byte)(0x0ff - (data[i] & 0xff));
+            }
+        }
         if ((y > 7) && (y%8 == 0)) {  // Every 8 scanlines
             cbLock.lock();
             try {
