@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -27,6 +27,7 @@ package java.lang.invoke;
 
 import jdk.internal.perf.PerfCounter;
 import jdk.internal.vm.annotation.DontInline;
+import jdk.internal.vm.annotation.Hidden;
 import jdk.internal.vm.annotation.Stable;
 import sun.invoke.util.Wrapper;
 
@@ -39,7 +40,7 @@ import java.util.Arrays;
 import java.util.HashMap;
 
 import static java.lang.invoke.LambdaForm.BasicType.*;
-import static java.lang.invoke.MethodHandleNatives.Constants.REF_invokeStatic;
+import static java.lang.invoke.MethodHandleNatives.Constants.*;
 import static java.lang.invoke.MethodHandleStatics.*;
 
 /**
@@ -186,50 +187,20 @@ class LambdaForm {
             return ALL_TYPES[type];
         }
         static BasicType basicType(char type) {
-            switch (type) {
-                case 'L': return L_TYPE;
-                case 'I': return I_TYPE;
-                case 'J': return J_TYPE;
-                case 'F': return F_TYPE;
-                case 'D': return D_TYPE;
-                case 'V': return V_TYPE;
+            return switch (type) {
+                case 'L' -> L_TYPE;
+                case 'I' -> I_TYPE;
+                case 'J' -> J_TYPE;
+                case 'F' -> F_TYPE;
+                case 'D' -> D_TYPE;
+                case 'V' -> V_TYPE;
                 // all subword types are represented as ints
-                case 'Z':
-                case 'B':
-                case 'S':
-                case 'C':
-                    return I_TYPE;
-                default:
-                    throw newInternalError("Unknown type char: '"+type+"'");
-            }
-        }
-        static BasicType basicType(Wrapper type) {
-            char c = type.basicTypeChar();
-            return basicType(c);
+                case 'Z', 'B', 'S', 'C' -> I_TYPE;
+                default -> throw newInternalError("Unknown type char: '" + type + "'");
+            };
         }
         static BasicType basicType(Class<?> type) {
-            if (!type.isPrimitive())  return L_TYPE;
-            return basicType(Wrapper.forPrimitiveType(type));
-        }
-        static BasicType[] basicTypes(String types) {
-            BasicType[] btypes = new BasicType[types.length()];
-            for (int i = 0; i < btypes.length; i++) {
-                btypes[i] = basicType(types.charAt(i));
-            }
-            return btypes;
-        }
-        static String basicTypeDesc(BasicType[] types) {
-            if (types == null) {
-                return null;
-            }
-            if (types.length == 0) {
-                return "";
-            }
-            StringBuilder sb = new StringBuilder();
-            for (BasicType bt : types) {
-                sb.append(bt.basicTypeChar());
-            }
-            return sb.toString();
+            return basicType(Wrapper.basicTypeChar(type));
         }
         static int[] basicTypeOrds(BasicType[] types) {
             if (types == null) {
@@ -246,10 +217,10 @@ class LambdaForm {
             return basicType(type).btChar;
         }
 
-        static byte[] basicTypesOrd(Class<?>[] types) {
-            byte[] ords = new byte[types.length];
+        static int[] basicTypesOrd(Class<?>[] types) {
+            int[] ords = new int[types.length];
             for (int i = 0; i < ords.length; i++) {
-                ords[i] = (byte)basicType(types[i]).ordinal();
+                ords[i] = basicType(types[i]).ordinal();
             }
             return ords;
         }
@@ -333,7 +304,9 @@ class LambdaForm {
         GET_DOUBLE_VOLATILE("getDoubleVolatile"),
         PUT_DOUBLE_VOLATILE("putDoubleVolatile"),
         TRY_FINALLY("tryFinally"),
+        TABLE_SWITCH("tableSwitch"),
         COLLECT("collect"),
+        COLLECTOR("collector"),
         CONVERT("convert"),
         SPREAD("spread"),
         LOOP("loop"),
@@ -387,28 +360,9 @@ class LambdaForm {
     LambdaForm(int arity, Name[] names, Kind kind) {
         this(arity, names, LAST_RESULT, /*forceInline=*/true, /*customized=*/null, kind);
     }
-    LambdaForm(int arity, Name[] names, boolean forceInline) {
-        this(arity, names, LAST_RESULT, forceInline, /*customized=*/null, Kind.GENERIC);
-    }
     LambdaForm(int arity, Name[] names, boolean forceInline, Kind kind) {
         this(arity, names, LAST_RESULT, forceInline, /*customized=*/null, kind);
     }
-    LambdaForm(Name[] formals, Name[] temps, Name result) {
-        this(formals.length, buildNames(formals, temps, result), LAST_RESULT, /*forceInline=*/true, /*customized=*/null);
-    }
-    LambdaForm(Name[] formals, Name[] temps, Name result, boolean forceInline) {
-        this(formals.length, buildNames(formals, temps, result), LAST_RESULT, forceInline, /*customized=*/null);
-    }
-
-    private static Name[] buildNames(Name[] formals, Name[] temps, Name result) {
-        int arity = formals.length;
-        int length = arity + temps.length + (result == null ? 0 : 1);
-        Name[] names = Arrays.copyOf(formals, length);
-        System.arraycopy(temps, 0, names, arity, temps.length);
-        if (result != null)
-            names[length - 1] = result;
-        return names;
-    }
 
     private LambdaForm(MethodType mt) {
         // Make a blank lambda form, which returns a constant zero or null.
@@ -500,6 +454,9 @@ class LambdaForm {
 
     /** Customize LambdaForm for a particular MethodHandle */
     LambdaForm customize(MethodHandle mh) {
+        if (customized == mh) {
+            return this;
+        }
         LambdaForm customForm = new LambdaForm(arity, names, result, forceInline, mh, kind);
         if (COMPILE_THRESHOLD >= 0 && isCompiled) {
             // If shared LambdaForm has been compiled, compile customized version as well.
@@ -593,8 +550,7 @@ class LambdaForm {
             Name n = names[i];
             assert(n.index() == i);
             for (Object arg : n.arguments) {
-                if (arg instanceof Name) {
-                    Name n2 = (Name) arg;
+                if (arg instanceof Name n2) {
                     int i2 = n2.index;
                     assert(0 <= i2 && i2 < names.length) : n.debugString() + ": 0 <= i2 && i2 < names.length: 0 <= " + i2 + " < " + names.length;
                     assert(names[i2] == n2) : Arrays.asList("-1-", i, "-2-", n.debugString(), "-3-", i2, "-4-", n2.debugString(), "-5-", names[i2].debugString(), "-6-", this);
@@ -625,9 +581,8 @@ class LambdaForm {
 
     /** Report the N-th argument name. */
     Name parameter(int n) {
-        assert(n < arity);
         Name param = names[n];
-        assert(param.isParam());
+        assert(n < arity && param.isParam());
         return param;
     }
 
@@ -652,7 +607,7 @@ class LambdaForm {
         for (int i = 0; i < arity; ++i) {
             ptypes[i] = parameterType(i).btClass;
         }
-        return MethodType.makeImpl(returnType().btClass, ptypes, true);
+        return MethodType.methodType(returnType().btClass, ptypes, true);
     }
 
     /** Return ABC_Z, where the ABC are parameter type characters, and Z is the return type character. */
@@ -666,9 +621,6 @@ class LambdaForm {
         assert(isValidSignature(sig));
         return sig.indexOf('_');
     }
-    static BasicType signatureReturn(String sig) {
-        return basicType(sig.charAt(signatureArity(sig) + 1));
-    }
     static boolean isValidSignature(String sig) {
         int arity = sig.indexOf('_');
         if (arity < 0)  return false;  // must be of the form *_*
@@ -683,16 +635,6 @@ class LambdaForm {
         }
         return true;  // [LIJFD]*_[LIJFDV]
     }
-    static MethodType signatureType(String sig) {
-        Class<?>[] ptypes = new Class<?>[signatureArity(sig)];
-        for (int i = 0; i < ptypes.length; i++)
-            ptypes[i] = basicType(sig.charAt(i)).btClass;
-        Class<?> rtype = signatureReturn(sig).btClass;
-        return MethodType.makeImpl(rtype, ptypes, true);
-    }
-    static MethodType basicMethodType(MethodType mt) {
-        return signatureType(basicTypeSignature(mt));
-    }
 
     /**
      * Check if i-th name is a call to MethodHandleImpl.selectAlternative.
@@ -746,6 +688,32 @@ class LambdaForm {
         return isMatchingIdiom(pos, "tryFinally", 2);
     }
 
+    /**
+     * Check if i-th name is a start of the tableSwitch idiom.
+     */
+    boolean isTableSwitch(int pos) {
+        // tableSwitch idiom:
+        //   t_{n}:L=MethodHandle.invokeBasic(...)     // args
+        //   t_{n+1}:L=MethodHandleImpl.tableSwitch(*, *, *, t_{n})
+        //   t_{n+2}:?=MethodHandle.invokeBasic(*, t_{n+1})
+        if (pos + 2 >= names.length)  return false;
+
+        final int POS_COLLECT_ARGS = pos;
+        final int POS_TABLE_SWITCH = pos + 1;
+        final int POS_UNBOX_RESULT = pos + 2;
+
+        Name collectArgs = names[POS_COLLECT_ARGS];
+        Name tableSwitch = names[POS_TABLE_SWITCH];
+        Name unboxResult = names[POS_UNBOX_RESULT];
+        return tableSwitch.refersTo(MethodHandleImpl.class, "tableSwitch") &&
+                collectArgs.isInvokeBasic() &&
+                unboxResult.isInvokeBasic() &&
+                tableSwitch.lastUseIndex(collectArgs) == 3 &&     // t_{n+1}:L=MethodHandleImpl.<invoker>(*, *, *, t_{n});
+                lastUseIndex(collectArgs) == POS_TABLE_SWITCH &&  // t_{n} is local: used only in t_{n+1}
+                unboxResult.lastUseIndex(tableSwitch) == 1 &&     // t_{n+2}:?=MethodHandle.invokeBasic(*, t_{n+1})
+                lastUseIndex(tableSwitch) == POS_UNBOX_RESULT;    // t_{n+1} is local: used only in t_{n+2}
+    }
+
     /**
      * Check if i-th name is a start of the loop idiom.
      */
@@ -800,7 +768,7 @@ class LambdaForm {
      * (a) redundant compilation work and (b) extra I$ pressure.
      * To control repeated versions, we need to be ready to
      * erase details from LFs and move them into MH data,
-     * whevener those details are not relevant to significant
+     * whenever those details are not relevant to significant
      * optimization.  "Significant" means optimization of
      * code that is actually hot.
      *
@@ -894,7 +862,7 @@ class LambdaForm {
     private static boolean argumentTypesMatch(String sig, Object[] av) {
         int arity = signatureArity(sig);
         assert(av.length == arity) : "av.length == arity: av.length=" + av.length + ", arity=" + arity;
-        assert(av[0] instanceof MethodHandle) : "av[0] not instace of MethodHandle: " + av[0];
+        assert(av[0] instanceof MethodHandle) : "av[0] not instance of MethodHandle: " + av[0];
         MethodHandle mh = (MethodHandle) av[0];
         MethodType mt = mh.type();
         assert(mt.parameterCount() == arity-1);
@@ -1051,12 +1019,18 @@ class LambdaForm {
     }
 
     public String toString() {
+        return debugString(-1);
+    }
+
+    String debugString(int indentLevel) {
+        String prefix = MethodHandle.debugPrefix(indentLevel);
         String lambdaName = lambdaName();
-        StringBuilder buf = new StringBuilder(lambdaName + "=Lambda(");
+        StringBuilder buf = new StringBuilder(lambdaName);
+        buf.append("=Lambda(");
         for (int i = 0; i < names.length; i++) {
             if (i == arity)  buf.append(")=>{");
             Name n = names[i];
-            if (i >= arity)  buf.append("\n    ");
+            if (i >= arity)  buf.append("\n    ").append(prefix);
             buf.append(n.paramString());
             if (i < arity) {
                 if (i+1 < arity)  buf.append(",");
@@ -1105,25 +1079,14 @@ class LambdaForm {
     static class NamedFunction {
         final MemberName member;
         private @Stable MethodHandle resolvedHandle;
-        @Stable MethodHandle invoker;
-        private final MethodHandleImpl.Intrinsic intrinsicName;
+        private @Stable MethodType type;
 
         NamedFunction(MethodHandle resolvedHandle) {
-            this(resolvedHandle.internalMemberName(), resolvedHandle, MethodHandleImpl.Intrinsic.NONE);
-        }
-        NamedFunction(MethodHandle resolvedHandle, MethodHandleImpl.Intrinsic intrinsic) {
-            this(resolvedHandle.internalMemberName(), resolvedHandle, intrinsic);
+            this(resolvedHandle.internalMemberName(), resolvedHandle);
         }
         NamedFunction(MemberName member, MethodHandle resolvedHandle) {
-            this(member, resolvedHandle, MethodHandleImpl.Intrinsic.NONE);
-        }
-        NamedFunction(MemberName member, MethodHandle resolvedHandle, MethodHandleImpl.Intrinsic intrinsic) {
             this.member = member;
             this.resolvedHandle = resolvedHandle;
-            this.intrinsicName = intrinsic;
-            assert(resolvedHandle == null ||
-                   resolvedHandle.intrinsicName() == MethodHandleImpl.Intrinsic.NONE ||
-                   resolvedHandle.intrinsicName() == intrinsic) : resolvedHandle.intrinsicName() + " != " + intrinsic;
              // The following assert is almost always correct, but will fail for corner cases, such as PrivateInvokeTest.
              //assert(!isInvokeBasic(member));
         }
@@ -1136,7 +1099,6 @@ class LambdaForm {
                 // necessary to pass BigArityTest
                 this.member = Invokers.invokeBasicMethod(basicInvokerType);
             }
-            this.intrinsicName = MethodHandleImpl.Intrinsic.NONE;
             assert(isInvokeBasic(member));
         }
 
@@ -1172,9 +1134,9 @@ class LambdaForm {
         public boolean equals(Object other) {
             if (this == other) return true;
             if (other == null) return false;
-            if (!(other instanceof NamedFunction)) return false;
-            NamedFunction that = (NamedFunction) other;
-            return this.member != null && this.member.equals(that.member);
+            return (other instanceof NamedFunction that)
+                    && this.member != null
+                    && this.member.equals(that.member);
         }
 
         @Override
@@ -1213,10 +1175,6 @@ class LambdaForm {
             Object rval;
             try {
                 traceInterpreter("[ call", this, arguments);
-                if (invoker == null) {
-                    traceInterpreter("| getInvoker", this);
-                    invoker();
-                }
                 // resolvedHandle might be uninitialized, ok for tracing
                 if (resolvedHandle == null) {
                     traceInterpreter("| resolve", this);
@@ -1232,17 +1190,24 @@ class LambdaForm {
         }
 
         private MethodHandle invoker() {
-            if (invoker != null)  return invoker;
-            // Get an invoker and cache it.
-            return invoker = computeInvoker(methodType().form());
+            return computeInvoker(methodType().form());
         }
 
         MethodType methodType() {
-            if (resolvedHandle != null)
+            MethodType type = this.type;
+            if (type == null) {
+                this.type = type = calculateMethodType(member, resolvedHandle);
+            }
+            return type;
+        }
+
+        private static MethodType calculateMethodType(MemberName member, MethodHandle resolvedHandle) {
+            if (resolvedHandle != null) {
                 return resolvedHandle.type();
-            else
+            } else {
                 // only for certain internal LFs during bootstrapping
                 return member.getInvocationType();
+            }
         }
 
         MemberName member() {
@@ -1289,7 +1254,15 @@ class LambdaForm {
         }
 
         public MethodHandleImpl.Intrinsic intrinsicName() {
-            return intrinsicName;
+            return resolvedHandle != null
+                ? resolvedHandle.intrinsicName()
+                : MethodHandleImpl.Intrinsic.NONE;
+        }
+
+        public Object intrinsicData() {
+            return resolvedHandle != null
+                ? resolvedHandle.intrinsicData()
+                : null;
         }
     }
 
@@ -1323,7 +1296,7 @@ class LambdaForm {
             if (c1 != NO_CHAR && !('A' <= c1 && c1 <= 'Z')) {
                 // wrong kind of char; bail out here
                 if (buf != null) {
-                    buf.append(signature.substring(i - c1reps, len));
+                    buf.append(signature, i - c1reps, len);
                 }
                 break;
             }
@@ -1355,13 +1328,15 @@ class LambdaForm {
         final Object constraint;  // additional type information, if not null
         @Stable final Object[] arguments;
 
+        private static final Object[] EMPTY_ARGS = new Object[0];
+
         private Name(int index, BasicType type, NamedFunction function, Object[] arguments) {
             this.index = (short)index;
             this.type = type;
             this.function = function;
             this.arguments = arguments;
             this.constraint = null;
-            assert(this.index == index);
+            assert(this.index == index && typesMatch(function, this.arguments));
         }
         private Name(Name that, Object constraint) {
             this.index = that.index;
@@ -1382,9 +1357,17 @@ class LambdaForm {
         Name(MemberName function, Object... arguments) {
             this(new NamedFunction(function), arguments);
         }
+        Name(NamedFunction function) {
+            this(-1, function.returnType(), function, EMPTY_ARGS);
+        }
+        Name(NamedFunction function, Object arg) {
+            this(-1, function.returnType(), function, new Object[] { arg });
+        }
+        Name(NamedFunction function, Object arg0, Object arg1) {
+            this(-1, function.returnType(), function, new Object[] { arg0, arg1 });
+        }
         Name(NamedFunction function, Object... arguments) {
-            this(-1, function.returnType(), function, arguments = Arrays.copyOf(arguments, arguments.length, Object[].class));
-            assert(typesMatch(function, arguments));
+            this(-1, function.returnType(), function, Arrays.copyOf(arguments, arguments.length, Object[].class));
         }
         /** Create a raw parameter of the given type, with an expected index. */
         Name(int index, BasicType type) {
@@ -1406,11 +1389,6 @@ class LambdaForm {
             return type.btChar;
         }
 
-        void resolve() {
-            if (function != null)
-                function.resolve();
-        }
-
         Name newIndex(int i) {
             if (initIndex(i))  return this;
             return cloneWithIndex(i);
@@ -1451,8 +1429,7 @@ class LambdaForm {
             boolean replaced = false;
         eachArg:
             for (int j = 0; j < arguments.length; j++) {
-                if (arguments[j] instanceof Name) {
-                    Name n = (Name) arguments[j];
+                if (arguments[j] instanceof Name n) {
                     int check = n.index;
                     // harmless check to see if the thing is already in newNames:
                     if (check >= 0 && check < newNames.length && n == newNames[check])
@@ -1479,8 +1456,7 @@ class LambdaForm {
             @SuppressWarnings("LocalVariableHidesMemberVariable")
             Object[] arguments = this.arguments;
             for (int j = 0; j < arguments.length; j++) {
-                if (arguments[j] instanceof Name) {
-                    Name n = (Name) arguments[j];
+                if (arguments[j] instanceof Name n) {
                     if (n.isParam() && n.index < INTERNED_ARGUMENT_LIMIT)
                         arguments[j] = internArgument(n);
                 }
@@ -1558,6 +1534,10 @@ class LambdaForm {
         }
 
         private boolean typesMatch(NamedFunction function, Object ... arguments) {
+            if (arguments == null) {
+                assert(function == null);
+                return true;
+            }
             assert(arguments.length == function.arity()) : "arity mismatch: arguments.length=" + arguments.length + " == function.arity()=" + function.arity() + " in " + debugString();
             for (int i = 0; i < arguments.length; i++) {
                 assert (typesMatch(function.parameterType(i), arguments[i])) : "types don't match: function.parameterType(" + i + ")=" + function.parameterType(i) + ", arguments[" + i + "]=" + arguments[i] + " in " + debugString();
@@ -1594,18 +1574,17 @@ class LambdaForm {
          *  Return 0 if the name is not used.
          */
         int useCount(Name n) {
-            if (arguments == null)  return 0;
             int count = 0;
-            for (int i = arguments.length; --i >= 0; ) {
-                if (arguments[i] == n)  ++count;
+            if (arguments != null) {
+                for (Object argument : arguments) {
+                    if (argument == n) {
+                        count++;
+                    }
+                }
             }
             return count;
         }
 
-        boolean contains(Name n) {
-            return this == n || lastUseIndex(n) >= 0;
-        }
-
         public boolean equals(Name that) {
             if (this == that)  return true;
             if (isParam())
@@ -1645,15 +1624,10 @@ class LambdaForm {
 
     /** Return the number of times n is used as an argument or return value. */
     int useCount(Name n) {
-        int nmax = names.length;
-        int end = lastUseIndex(n);
-        if (end < 0)  return 0;
-        int count = 0;
-        if (end == nmax) { count++; end--; }
-        int beg = n.index() + 1;
-        if (beg < arity)  beg = arity;
-        for (int i = beg; i <= end; i++) {
-            count += names[i].useCount(n);
+        int count = (result == n.index) ? 1 : 0;
+        int i = Math.max(n.index + 1, arity);
+        while (i < names.length) {
+            count += names[i++].useCount(n);
         }
         return count;
     }
@@ -1757,10 +1731,10 @@ class LambdaForm {
             MemberName idMem = new MemberName(LambdaForm.class, "identity_"+btChar, idType, REF_invokeStatic);
             MemberName zeMem = null;
             try {
-                idMem = IMPL_NAMES.resolveOrFail(REF_invokeStatic, idMem, null, NoSuchMethodException.class);
+                idMem = IMPL_NAMES.resolveOrFail(REF_invokeStatic, idMem, null, LM_TRUSTED, NoSuchMethodException.class);
                 if (!isVoid) {
                     zeMem = new MemberName(LambdaForm.class, "zero_"+btChar, zeType, REF_invokeStatic);
-                    zeMem = IMPL_NAMES.resolveOrFail(REF_invokeStatic, zeMem, null, NoSuchMethodException.class);
+                    zeMem = IMPL_NAMES.resolveOrFail(REF_invokeStatic, zeMem, null, LM_TRUSTED, NoSuchMethodException.class);
                 }
             } catch (IllegalAccessException|NoSuchMethodException ex) {
                 throw newInternalError(ex);
@@ -1784,15 +1758,15 @@ class LambdaForm {
                 Name[] idNames = new Name[] { argument(0, L_TYPE), argument(1, type) };
                 idForm = new LambdaForm(2, idNames, 1, Kind.IDENTITY);
                 idForm.compileToBytecode();
-                idFun = new NamedFunction(idMem, SimpleMethodHandle.make(idMem.getInvocationType(), idForm),
-                            MethodHandleImpl.Intrinsic.IDENTITY);
+                idFun = new NamedFunction(idMem, MethodHandleImpl.makeIntrinsic(SimpleMethodHandle.make(idMem.getInvocationType(), idForm),
+                            MethodHandleImpl.Intrinsic.IDENTITY));
 
                 Object zeValue = Wrapper.forBasicType(btChar).zero();
                 Name[] zeNames = new Name[] { argument(0, L_TYPE), new Name(idFun, zeValue) };
                 zeForm = new LambdaForm(1, zeNames, 1, Kind.ZERO);
                 zeForm.compileToBytecode();
-                zeFun = new NamedFunction(zeMem, SimpleMethodHandle.make(zeMem.getInvocationType(), zeForm),
-                        MethodHandleImpl.Intrinsic.ZERO);
+                zeFun = new NamedFunction(zeMem, MethodHandleImpl.makeIntrinsic(SimpleMethodHandle.make(zeMem.getInvocationType(), zeForm),
+                        MethodHandleImpl.Intrinsic.ZERO));
             }
 
             LF_zero[ord] = zeForm;
@@ -1828,15 +1802,6 @@ class LambdaForm {
     @interface Compiled {
     }
 
-    /**
-     * Internal marker for LambdaForm interpreter frames.
-     */
-    /*non-public*/
-    @Target(ElementType.METHOD)
-    @Retention(RetentionPolicy.RUNTIME)
-    @interface Hidden {
-    }
-
     private static final HashMap<String,Integer> DEBUG_NAME_COUNTERS;
     private static final HashMap<LambdaForm,String> DEBUG_NAMES;
     static {
