@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -382,7 +382,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       Formatter buf = new Formatter(genHTML);
       buf.append(genMethodLink(m));
       buf.append(" of ");
-      buf.append(genKlassLink((InstanceKlass) m.getMethodHolder()));
+      buf.append(genKlassLink(m.getMethodHolder()));
       return buf.toString();
    }
 
@@ -453,13 +453,13 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       buf.headerCell("Constant Value");
       buf.endTag("tr");
 
-      final int length = (int) cpool.getLength();
+      final int length = cpool.getLength();
       // zero'th pool entry is always invalid. ignore it.
       for (int index = 1; index < length; index++) {
          buf.beginTag("tr");
          buf.cell(Integer.toString(index));
 
-         int ctag = (int) cpool.getTags().at((int) index);
+         int ctag = cpool.getTags().at(index);
          switch (ctag) {
             case JVM_CONSTANT_Integer:
                buf.cell("JVM_CONSTANT_Integer");
@@ -497,7 +497,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
 
             case JVM_CONSTANT_Class:
                buf.cell("JVM_CONSTANT_Class");
-               Klass klass = (Klass) cpool.getKlassAt(index);
+               Klass klass = cpool.getKlassAt(index);
                if (klass instanceof InstanceKlass) {
                   buf.cell(genKlassLink((InstanceKlass) klass));
                } else {
@@ -586,7 +586,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
          Formatter buf = new Formatter(genHTML);
          buf.genHTMLPrologue(genConstantPoolTitle(cpool));
          buf.h3("Holder Class");
-         buf.append(genKlassLink((InstanceKlass) cpool.getPoolHolder()));
+         buf.append(genKlassLink(cpool.getPoolHolder()));
          buf.h3("Constants");
          buf.append(genHTMLTableForConstantPool(cpool));
          buf.genHTMLEpilogue();
@@ -603,7 +603,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
    protected String genConstantPoolTitle(ConstantPool cpool) {
       Formatter buf = new Formatter(genHTML);
       buf.append("Constant Pool of [");
-      buf.append(genKlassTitle((InstanceKlass) cpool.getPoolHolder()));
+      buf.append(genKlassTitle(cpool.getPoolHolder()));
       buf.append("] @");
       buf.append(cpool.getAddress().toString());
       return buf.toString();
@@ -621,7 +621,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
          buf.genHTMLPrologue(genMethodTitle(method));
 
          buf.h3("Holder Class");
-         buf.append(genKlassLink((InstanceKlass) method.getMethodHolder()));
+         buf.append(genKlassLink(method.getMethodHolder()));
 
          NMethod nmethod = method.getNativeMethod();
          if (nmethod != null) {
@@ -630,7 +630,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
          }
 
          boolean hasThrows = method.hasCheckedExceptions();
-         ConstantPool cpool = ((InstanceKlass) method.getMethodHolder()).getConstants();
+         ConstantPool cpool = method.getMethodHolder().getConstants();
          if (hasThrows) {
             buf.h3("Checked Exception(s)");
             CheckedExceptionElement[] exceptions = method.getCheckedExceptions();
@@ -675,9 +675,9 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
                              buf.beginTag("tr");
                              if (hasLineNumbers) {
                                 int lineNumber = method.getLineNumberFromBCI(curBci);
-                                buf.cell(Integer.toString(lineNumber) + spaces);
+                                buf.cell(lineNumber + spaces);
                              }
-                             buf.cell(Integer.toString(curBci) + spaces);
+                             buf.cell(curBci + spaces);
 
                              buf.beginTag("td");
                              String instrStr = null;
@@ -708,7 +708,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
                                   if (m != null) {
                                     buf.link(genMethodHref(m), instrStr);
                                     buf.append(" of ");
-                                    InstanceKlass klass = (InstanceKlass) m.getMethodHolder();
+                                    InstanceKlass klass = m.getMethodHolder();
                                     buf.link(genKlassHref(klass), genKlassTitle(klass));
                                   } else {
                                     buf.append(instrStr);
@@ -815,7 +815,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       CodeBlob blob = null;
 
       try {
-         blob = (CodeBlob)VM.getVM().getCodeCache().findBlobUnsafe(pc);
+         blob = VM.getVM().getCodeCache().findBlobUnsafe(pc);
       } catch (Exception exp) {
          // ignore
       }
@@ -883,10 +883,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
    }
 
    protected String genMultPCHref(String pcs) {
-      StringBuffer buf = new StringBuffer(genBaseHref());
-      buf.append("pc_multiple=");
-      buf.append(pcs);
-      return buf.toString();
+      return genBaseHref() + "pc_multiple=" + pcs;
    }
 
    protected String genPCHref(Address addr) {
@@ -952,7 +949,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       }
 
       public void endInstruction(long endPc) {
-         instrSize += endPc - pc;
+         instrSize += (int) (endPc - pc);
          if (genHTML) buf.br();
 
          if (nmethod != null) {
@@ -986,7 +983,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
 
       public void epilogue() {
       }
-   };
+   }
 
    protected String genHTMLForRawDisassembly(sun.jvm.hotspot.debugger.Address addr,
                                              int size,
@@ -1010,7 +1007,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
          Formatter tmpBuf = new Formatter(genHTML);
          long startPc = addressToLong(addr);
          tmpBuf.append("0x");
-         tmpBuf.append(Long.toHexString(startPc + visitor.getInstructionSize()).toString());
+         tmpBuf.append(Long.toHexString(startPc + visitor.getInstructionSize()));
          tmpBuf.append(",0x");
          tmpBuf.append(Long.toHexString(startPc));
          if (prevPCs != null) {
@@ -1067,21 +1064,21 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
             buf.append(Integer.toString(line));
         }
 
-        List locals = sd.getLocals();
+        List<ScopeValue> locals = sd.getLocals();
         if (locals != null) {
             buf.br();
             buf.append(tabs);
             buf.append(genHTMLForLocals(sd, locals));
         }
 
-        List expressions = sd.getExpressions();
+        List<ScopeValue> expressions = sd.getExpressions();
         if (expressions != null) {
             buf.br();
             buf.append(tabs);
             buf.append(genHTMLForExpressions(sd, expressions));
         }
 
-        List monitors = sd.getMonitors();
+        List<MonitorValue> monitors = sd.getMonitors();
         if (monitors != null) {
             buf.br();
             buf.append(tabs);
@@ -1097,14 +1094,14 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
             return;
         }
 
-        List objects = sd.getObjects();
+        List<ObjectValue> objects = sd.getObjects();
         if (objects == null) {
             return;
         }
         int length = objects.size();
         for (int i = 0; i < length; i++) {
             buf.append(tabs);
-            ObjectValue ov = (ObjectValue)objects.get(i);
+            ObjectValue ov = objects.get(i);
             buf.append("ScObj" + i);
             ScopeValue sv = ov.getKlass();
             if (Assert.ASSERTS_ENABLED) {
@@ -1124,7 +1121,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
                 int flen = ov.fieldsSize();
 
                 U2Array klfields = kls.getFields();
-                int klen = (int) klfields.length();
+                int klen = klfields.length();
                 int findex = 0;
                 for (int index = 0; index < klen; index++) {
                     int accsFlags = kls.getFieldAccessFlags(index);
@@ -1183,7 +1180,8 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       Formatter buf = new Formatter(genHTML);
 
       final class OopMapValueIterator {
-         final Formatter iterate(OopMapStream oms, String type, boolean printContentReg) {
+         final Formatter iterate(OopMapStream oms, String type, boolean printContentReg,
+                                 OopMapValue.OopTypes filter) {
             Formatter tmpBuf = new Formatter(genHTML);
             boolean found = false;
             tmpBuf.beginTag("tr");
@@ -1191,7 +1189,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
             tmpBuf.append(type);
             for (; ! oms.isDone(); oms.next()) {
                OopMapValue omv = oms.getCurrent();
-               if (omv == null) {
+               if (omv == null || omv.getType() != filter) {
                   continue;
                }
                found = true;
@@ -1227,17 +1225,21 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       buf.beginTable(0);
 
       OopMapValueIterator omvIterator = new OopMapValueIterator();
-      OopMapStream oms = new OopMapStream(map, OopMapValue.OopTypes.OOP_VALUE);
-      buf.append(omvIterator.iterate(oms, "Oops:", false));
+      OopMapStream oms = new OopMapStream(map);
+      buf.append(omvIterator.iterate(oms, "Oops:", false,
+                                     OopMapValue.OopTypes.OOP_VALUE));
 
-      oms = new OopMapStream(map, OopMapValue.OopTypes.NARROWOOP_VALUE);
-      buf.append(omvIterator.iterate(oms, "NarrowOops:", false));
+      oms = new OopMapStream(map);
+      buf.append(omvIterator.iterate(oms, "NarrowOops:", false,
+                                     OopMapValue.OopTypes.NARROWOOP_VALUE));
 
-      oms = new OopMapStream(map, OopMapValue.OopTypes.CALLEE_SAVED_VALUE);
-      buf.append(omvIterator.iterate(oms, "Callee saved:",  true));
+      oms = new OopMapStream(map);
+      buf.append(omvIterator.iterate(oms, "Callee saved:", true,
+                                     OopMapValue.OopTypes.CALLEE_SAVED_VALUE));
 
-      oms = new OopMapStream(map, OopMapValue.OopTypes.DERIVED_OOP_VALUE);
-      buf.append(omvIterator.iterate(oms, "Derived oops:", true));
+      oms = new OopMapStream(map);
+      buf.append(omvIterator.iterate(oms, "Derived oops:", true,
+                                     OopMapValue.OopTypes.DERIVED_OOP_VALUE));
 
       buf.endTag("table");
       return buf.toString();
@@ -1279,8 +1281,6 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
          if (w == Location.Where.ON_STACK) {
             buf.append("stack[" + loc.getStackOffset() + "]");
          } else if (w == Location.Where.IN_REGISTER) {
-            boolean isFloat = (type == Location.Type.FLOAT_IN_DBL ||
-                               type == Location.Type.DBL);
             int regNum = loc.getRegisterNumber();
             VMReg vmReg = new VMReg(regNum);
             buf.append(VMRegImpl.getRegisterName(vmReg.getValue()));
@@ -1352,12 +1352,12 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       return buf.toString();
    }
 
-   protected String genHTMLForScopeValues(ScopeDesc sd, boolean locals, List values) {
+   protected String genHTMLForScopeValues(ScopeDesc sd, boolean locals, List<ScopeValue> values) {
       int length = values.size();
       Formatter buf = new Formatter(genHTML);
       buf.append(locals? "locals " : "expressions ");
       for (int i = 0; i < length; i++) {
-         ScopeValue sv = (ScopeValue) values.get(i);
+         ScopeValue sv = values.get(i);
          if (sv == null) {
             continue;
          }
@@ -1387,20 +1387,20 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
       return buf.toString();
    }
 
-   protected String genHTMLForLocals(ScopeDesc sd, List locals) {
+   protected String genHTMLForLocals(ScopeDesc sd, List<ScopeValue> locals) {
       return genHTMLForScopeValues(sd, true, locals);
    }
 
-   protected String genHTMLForExpressions(ScopeDesc sd, List expressions) {
+   protected String genHTMLForExpressions(ScopeDesc sd, List<ScopeValue> expressions) {
       return genHTMLForScopeValues(sd, false, expressions);
    }
 
-   protected String genHTMLForMonitors(ScopeDesc sd, List monitors) {
+   protected String genHTMLForMonitors(ScopeDesc sd, List<MonitorValue> monitors) {
       int length = monitors.size();
       Formatter buf = new Formatter(genHTML);
       buf.append("monitors ");
       for (int i = 0; i < length; i++) {
-         MonitorValue mv = (MonitorValue) monitors.get(i);
+         MonitorValue mv = monitors.get(i);
          if (mv == null) {
             continue;
          }
@@ -1542,7 +1542,7 @@ public class HTMLGenerator implements /* imports */ ClassConstants {
    }
 
    protected String genDumpKlassesHref(InstanceKlass[] klasses) {
-      StringBuffer buf = new StringBuffer(genBaseHref());
+      StringBuilder buf = new StringBuilder(genBaseHref());
       buf.append("jcore_multiple=");
       for (int k = 0; k < klasses.length; k++) {
          buf.append(klasses[k].getAddress().toString());
